# encoding: UTF-8
# typed: true

# Persistent-💎: Ruby gem for easily creating immutable data structures
# Copyright (c) 2017 Ivo Anjo <ivo.anjo@ist.utl.pt>
#
# This file is part of Persistent-💎.
#
# MIT License
#
# Copyright (c) 2017 Ivo Anjo
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# frozen_string_literal: true

module Persistent💎
  module Ruby19And20Support
    # Enumerable#to_h was only introduced in Ruby 2.1 so we have to do it by hand on 2.0/1.9
    if RUBY_VERSION.start_with?('2.0.') || RUBY_VERSION.start_with?('1.9.')
      def self.enumerable_to_h(enumerable)
        enumerable.each_with_object({}) do |(key, value), result|
          result[key] = value
        end
      end
    else
      def self.enumerable_to_h(enumerable)
        enumerable.to_h
      end
    end
  end
end
